# -*- coding: utf-8 -*-

import hashlib

def sort_hash_urlencode(d, pop_keys=None):
    """对字典进行排序, 按照key的顺序从小到大进行排列"""
    d2 = d.copy()
    # 剔除不参与排序的key
    if pop_keys is not None:
        for k in pop_keys:
            if k in d2:
                d2.pop(k)
    return "&".join([
        "%s=%s" % i for i in 
        sorted(d2.items(), key=lambda m:m[0])
    ])

def create_hexdigest_sign(data={}, secret_key="", sign_key="sign"):
    """对请求参数进行签名"""
    if secret_key == "":
        raise Exception("secret_key is required.")
    if not data:
        raise Exception("data is required.")

    request_str = sort_hash_urlencode(data, pop_keys=(sign_key,))
    unsigned_str = "%s%s" % (request_str, secret_key)
    return hashlib.md5(unsigned_str).hexdigest()

def create_hexdigest_data(data={}, secret_key="", sign_key="sign"):
    """生成签名参数"""
    sign_val = create_hexdigest_sign(
        data=data, 
        secret_key=secret_key, 
        sign_key=sign_key)
    data.update({sign_key: sign_val})
    return data

def validate_hexdigest_data(data={}, secret_key="", sign_key="sign"):
    """验证签名"""
    if sign_key not in data:
        raise Exception("sign is required.")
    req_sign_val = data[sign_key]
    sign_val = create_hexdigest_sign(
        data=data, 
        secret_key=secret_key, 
        sign_key=sign_key)
    return req_sign_val == sign_val
