# -*- coding: utf-8 -*-

import re
import datetime
from decimal import Decimal

import xlwt

class CsvFile:
    """
    encode 文件编码
    output 导出文件路径 *
    """

    def __init__(self, encoding='utf-8', output=None):
        self.output = output

    def init(self, record):
        """
        record list对象
        """
        with open(self.output, "w") as f:
            for eachline in record:
                f.write(",".join(eachline)+"\r\n")

class XlsFile:
    """
    encode 文件编码
    output 导出文件路径 *

    """

    def __init__(self, encoding='utf-8', output=None, style_kwargs={}):
        self.workbook = xlwt.Workbook(encoding=encoding)
        self.output = output
        self.floatre = re.compile("^\d+\.\d{2}$")

        self.style_int = xlwt.XFStyle()
        self.style_int.num_format_str = '0'

        self.style_float = xlwt.XFStyle()
        self.style_float.num_format_str = style_kwargs.get("style_float", '0.00')

        self.style_normal = xlwt.XFStyle()
        self.style_normal.num_format_str = '@'

        self.style_date = xlwt.XFStyle()
        self.style_date.num_format_str = style_kwargs.get("style_date", 'YYYY-MM-DD')

    def init(self, record):
        """
        给一个字典, key是工作区, value是内容, value为list
        """
        for sheet, data in record.items():
            worksheet = self.workbook.add_sheet(sheet)
            for row_index, item in enumerate(data):
                for column_index, v in enumerate(item):
                    if isinstance(v, Decimal) or isinstance(v, float):
                        worksheet.write(row_index, column_index, 
                            label=float(v), style=self.style_float)
                    elif isinstance(v, int) or isinstance(v, long):
                        worksheet.write(row_index, column_index, 
                            label=int(v), style=self.style_int)
                    elif isinstance(v, datetime.date):
                        worksheet.write(row_index, column_index, 
                            label=v, style=self.style_date)
                    else:
                        worksheet.write(row_index, column_index, label=str(v))
        self.workbook.save(self.output)
